package eu.mhsl.craftattack.spawn.appliances.restart;

import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.appliances.restart.command.CancelRestartCommand;
import eu.mhsl.craftattack.spawn.appliances.restart.command.ScheduleRestartCommand;
import eu.mhsl.craftattack.spawn.util.Countdown;
import eu.mhsl.craftattack.spawn.util.IteratorUtil;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.jetbrains.annotations.NotNull;

import java.util.List;

public class Restart extends Appliance {
    private final Countdown countdown;

    public Restart() {
        this.countdown = new Countdown(60, this::format, this::announce, this::onDone);
    }

    public void scheduleRestart() {
        try {
            this.countdown.start();
        } catch (IllegalStateException e) {
            throw new ApplianceCommand.Error(e.getMessage());
        }
    }

    public void cancelRestart() {
        try {
            this.countdown.cancel();
        } catch (IllegalStateException e) {
            throw new ApplianceCommand.Error(e.getMessage());
        }
    }

    private Component format(Countdown.AnnouncementData data) {
        return Component.text()
                .append(Component.text("Serverneustart in ", NamedTextColor.DARK_RED))
                .append(Component.text(data.count(), NamedTextColor.RED))
                .append(Component.text(" " + data.unit() + "!", NamedTextColor.DARK_RED))
                .build();
    }

    private void announce(Component message) {
        IteratorUtil.onlinePlayers(player -> player.sendMessage(message));
    }

    private void onDone() {
        IteratorUtil.onlinePlayers(player -> {
            player.kick(
                    Component.text()
                            .appendNewline().appendNewline()
                            .append(Component.text("Serverneustart", NamedTextColor.DARK_RED)).appendNewline().appendNewline()
                            .append(Component.text("Wir sind gleich wieder online!", NamedTextColor.GOLD)).appendNewline()
                            .append(Component.text("Verbinde Dich dann erneut.", NamedTextColor.GRAY)).appendNewline()
                            .build()
            );
        });
        Bukkit.shutdown();
    }

    @Override
    protected @NotNull List<ApplianceCommand<?>> commands() {
        return List.of(new ScheduleRestartCommand(), new CancelRestartCommand());
    }
}
